package ami

import (
	"context"
	"errors"
	"fmt"
	"regexp"
	"strings"
	"time"

	"github.com/sirupsen/logrus"
	goami "gitlab.atsaero.ru/go-packages/goami/ami"
)

type Ami interface {
	MixMonitor(channelID, file string) error
	Monitor(channelID, file, format string, mix bool) error
	StopMonitor(channelID string) error
	Redirect(channelID, ext string) error
	Setvar(channelID, varl, value string) error
	Originate(uuid, channelID, ext, cnt, variables string, timeout int, clbFunc ...CallbackEventFunc) error
	Ping() error
}

type session struct {
	ctx    context.Context
	pool   ConnPool
	addr   string
	logger logrus.Entry
}

var ADDRTEMPLATE = regexp.MustCompile(`^[^:]+:[\d]+$`)

func NewSession(ctx context.Context, pool ConnPool, addr string, logger *logrus.Entry) (*session, error) {
	if !ADDRTEMPLATE.MatchString(addr) {
		return nil, fmt.Errorf("wrong address: '%s'", addr)
	}
	s := &session{ctx: ctx, pool: pool, addr: addr, logger: *logger.WithField("mode", "session")}
	return s, nil
}

func (s *session) Ping() error {
	commFunc := s.pool.launchCommands(s.addr)

	comm := func(socket *goami.Socket) error {
		uuid, err := goami.GetUUID()
		if err != nil {
			return err
		}

		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*500)
		defer cancel()

		err = goami.Ping(ctx, socket, uuid)
		if err != nil {
			s.logger.Errorf("ping failed: %v", err)
			return err
		}
		s.logger.Info("ping success")
		return nil
	}

	return commFunc(comm)
}

func (s *session) MixMonitor(channelID, file string) error {
	commFunc := s.pool.launchCommands(s.addr)

	comm := func(socket *goami.Socket) error {
		uuid, err := goami.GetUUID()
		if err != nil {
			return err
		}

		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*500)
		defer cancel()

		res, err := goami.MixMonitor(ctx, socket, uuid, channelID, file, "", "")
		if err != nil {
			s.logger.Errorf("MixMonitor failed: %v", err)
			return err
		}

		if strings.ToLower(res.Get("Response")) == "error" {
			msg := res.Get("Message")
			if msg == "" {
				s.logger.Error("MixMonitor failed: unknown error")
				return errors.New("unknown error")
			}
			s.logger.Errorf("MixMonitor failed: %v", msg)
			return errors.New(msg)
		}

		return nil
	}

	return commFunc(comm)
}

func (s *session) Monitor(channelID, file, format string, mix bool) error {
	commFunc := s.pool.launchCommands(s.addr)

	comm := func(socket *goami.Socket) error {
		uuid, err := goami.GetUUID()
		if err != nil {
			return err
		}

		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*500)
		defer cancel()

		res, err := goami.Monitor(ctx, socket, uuid, channelID, file, format, mix)
		if err != nil {
			s.logger.Errorf("Monitor failed: %v", err)
			return err
		}

		if strings.ToLower(res.Get("Response")) == "error" {
			msg := res.Get("Message")
			if msg == "" {
				s.logger.Error("Monitor failed: unknown error")
				return errors.New("unknown error")
			}
			s.logger.Errorf("Monitor failed: %v", msg)
			return errors.New(msg)
		}

		return nil
	}

	return commFunc(comm)

}

func (s *session) StopMonitor(channelID string) error {
	commFunc := s.pool.launchCommands(s.addr, nil)

	comm := func(socket *goami.Socket) error {
		uuid, err := goami.GetUUID()
		if err != nil {
			return err
		}

		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*500)
		defer cancel()

		res, err := goami.StopMonitor(ctx, socket, uuid, channelID)
		if err != nil {
			s.logger.Errorf("StopMonitor failed: %v", err)
			return err
		}

		if strings.ToLower(res.Get("Response")) == "error" {
			msg := res.Get("Message")
			if msg == "" {
				s.logger.Error("StopMonitor failed: unknown error")
				return errors.New("unknown error")
			}
			s.logger.Errorf("StopMonitor failed: %v", msg)
			return errors.New(msg)
		}

		return nil
	}

	return commFunc(comm)

}

func (s *session) Setvar(channelID, varl, value string) error {
	commFunc := s.pool.launchCommands(s.addr, nil)

	comm := func(socket *goami.Socket) error {
		uuid, err := goami.GetUUID()
		if err != nil {
			return err
		}

		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*500)
		defer cancel()

		res, err := goami.Setvar(ctx, socket, uuid, channelID, varl, value)
		if err != nil {
			s.logger.Errorf("ami.Setvar failed: %v", err)
			return err
		}

		if strings.ToLower(res.Get("Response")) == "error" {
			msg := res.Get("Message")
			if msg == "" {
				s.logger.Error("ami.Setvar failed: unknown error")
				return errors.New("unknown error")
			}
			s.logger.Errorf("ami.Setvar failed: %v", msg)
			return errors.New(msg)
		}

		return nil
	}

	return commFunc(comm)
}

func (s *session) Redirect(channelID, ext string) error {
	commFunc := s.pool.launchCommands(s.addr, nil)

	comm := func(socket *goami.Socket) error {
		uuid, err := goami.GetUUID()
		if err != nil {
			return err
		}

		ctx, cancel := context.WithTimeout(context.Background(), time.Millisecond*500)
		defer cancel()
		callData := goami.CallData{
			Channel:  channelID,
			Exten:    ext,
			Context:  "transfer",
			Priority: "1",
		}
		res, err := goami.Redirect(ctx, socket, uuid, callData)
		if err != nil {
			s.logger.Errorf("ami.Redirect failed: %v", err)
			return err
		}

		if strings.ToLower(res.Get("Response")) == "error" {
			msg := res.Get("Message")
			if msg == "" {
				s.logger.Error("ami.Redirect failed: unknown error")
				return errors.New("unknown error")
			}
			s.logger.Errorf("ami.Redirect failed: %v", msg)
			return errors.New(msg)
		}

		return nil
	}

	return commFunc(comm)
}

func (s *session) Originate(uuid, channelID, ext, cnt, variables string, timeout int, clbFunc ...CallbackEventFunc) error {

	if len(uuid) == 0 {
		return errors.New("uuid is empty")
	}

	callbacks := make([]*callbackParams, len(clbFunc))
	for i, f := range clbFunc {
		callbacks[i] = &callbackParams{
			key: uuid,
			f:   f,
		}
	}

	commFunc := s.pool.launchCommands(s.addr, callbacks...)

	comm := func(socket *goami.Socket) error {

		ctx, cancel := context.WithTimeout(context.Background(), time.Second*5)
		defer cancel()

		callData := goami.OriginateData{
			Channel:  channelID,
			Exten:    ext,
			Context:  cnt,
			Priority: 1,
			Timeout:  timeout,
			Account:  uuid,
			Async:    "true",
			Variable: variables,
		}

		res, err := goami.Originate(ctx, socket, uuid, callData)
		if err != nil {
			s.logger.Errorf("originate failed: %v", err)
			return err
		}

		if strings.ToLower(res.Get("Response")) == "error" {
			msg := res.Get("Message")
			if msg == "" {
				s.logger.Error("ami.Originate failed: unknown error")
				return errors.New("unknown error")
			}
			s.logger.Errorf("ami.Originate failed: %v", msg)
			return errors.New(msg)
		}

		return nil
	}

	return commFunc(comm)
}
