package ami

import (
	"bytes"
	"context"
	"fmt"
	"strings"
	"sync"
	"time"

	"github.com/sirupsen/logrus"
	goami "gitlab.atsaero.ru/go-packages/goami/ami"
)

const (
	EventDialBegin         = "DialBegin"
	EventDialEnd           = "DialEnd"
	EventHangup            = "Hangup"
	EventOriginateResponse = "OriginateResponse"
)

const (
	cleanePeriod       = time.Minute * 30 // период запуска cleaner
	callbackTimeToLive = time.Hour        // лимит жизни callback
	rights             = "call,all"
)

// CallbackEventFunc функция обработчика события
type CallbackEventFunc func(EventResponse) error

// callback элемент очереди обработчика события
type callback struct {
	f           CallbackEventFunc
	timeCreated time.Time
}

// event структура события
type event struct {
	key  string
	data EventResponse
}

type eventPool struct {
	sync.RWMutex
	callbacks map[string][]callback
	events    chan event
	ctx       context.Context
	addr      string
	user      string
	secret    string
	logger    *logrus.Entry
}

type EventResponse map[string][]string

func (er EventResponse) Get(key string) string {
	if er == nil {
		return ""
	}
	rs := er[key]
	if len(rs) == 0 {
		return ""
	}
	return rs[0]
}

func (er EventResponse) Event() string {
	return er.Get("Event")
}

func (er EventResponse) Key() string {
	return er.Get("KEY")
}

func NewEventPool(ctx context.Context, addr, user, secret string, logger *logrus.Entry) *eventPool {
	e := eventPool{
		ctx:       ctx,
		logger:    logger,
		callbacks: make(map[string][]callback),
		events:    make(chan event, 1000),
		addr:      addr,
		user:      user,
		secret:    secret,
	}

	// запуск обработчика событий
	go e.launchHandler()

	// запуск механизма удаления отработавших callbacks
	go func() {
		t := time.NewTicker(cleanePeriod)
	loop:
		for {
			select {
			case <-e.ctx.Done():
				break loop
			case <-t.C:
				e.clean()
			}
		}
	}()

	// запуск worker
	launchDuration := time.Millisecond
	go func() {
		for e.ctx.Err() == nil {
			time.Sleep(launchDuration)
			if err := e.worker(); err != nil {
				e.logger.Errorf("event worker for '%s' failed: %v", e.addr, err)
			}
			if launchDuration < time.Minute*5 {
				launchDuration += time.Second * 10
			}
		}
	}()

	return &e
}

func (e *eventPool) launchHandler() {
	e.logger.Debug("eventPool handler started")
	defer e.logger.Debug("eventPool handler stopped")

	for {
		select {
		case <-e.ctx.Done():
			return
		case event, ok := <-e.events:
			if !ok {
				return
			}
			e.RLock()
			calbacks, ok := e.callbacks[event.key]
			e.RUnlock()

			if ok {
				for _, c := range calbacks {
					go func(c callback) {
						if err := c.f(event.data); err != nil {
							e.logger.Errorf("ami event '%s' failed: %v", event.key, err)
						}
					}(c)
				}
			}

		}
	}
}

// clean очищает очередь calbacks от выполненных или протухших заданий
func (e *eventPool) clean() {
	keysForDel := make([]string, 0)
	e.RLock()
	for key, cbs := range e.callbacks {
		var done int
		for _, cb := range cbs {
			if time.Now().After(cb.timeCreated.Add(callbackTimeToLive)) {
				done += 1
			}
		}
		if done == len(cbs) {
			keysForDel = append(keysForDel, key)
		}
	}
	e.RUnlock()
	if len(keysForDel) > 0 {
		e.Lock()
		for _, key := range keysForDel {
			delete(e.callbacks, key)
		}
		e.Unlock()
		e.logger.Debugf("removed events: %v", keysForDel)
	}
}

func (e *eventPool) AddCallback(key string, f CallbackEventFunc) {
	e.Lock()
	defer e.Unlock()
	e.callbacks[key] = append(e.callbacks[key], callback{f: f, timeCreated: time.Now()})
}

func (e *eventPool) PushEvent(ev event) {
	select {
	case e.events <- ev:
	default:
		e.logger.Warnf("skip event '%s': chan overload", ev.key)
	}
}

func (e *eventPool) worker() error {
	e.logger.Debugf("start event-worker for %s", e.addr)
	defer e.logger.Debugf("stopped event-worker for %s", e.addr)

	ctx, cancel := context.WithCancel(e.ctx)
	defer cancel()

	connTimer := time.NewTimer(dialTimeout)
	connected := make(chan struct{})

	// реализация dialTimeout для ami.NewSocket
	go func() {
		select {
		case <-connTimer.C:
			cancel()
		case <-connected:
		}
	}()

	uuid, err := goami.GetUUID()
	if err != nil {
		return fmt.Errorf("ami.GetUUID failed: %v", err)
	}

	socket, err := goami.NewSocket(ctx, e.addr)
	if err != nil {
		return fmt.Errorf("ami.NewSocket failed: %v", err)
	}
	defer socket.Close(ctx)

	if err := goami.Login(ctx, socket, e.user, e.secret, rights, uuid); err != nil {
		return fmt.Errorf("ami.Login failed: %v", err)
	}

	close(connected)

	e.logger.Infof("event-worker for %s connected and ready", e.addr)

	var buffer bytes.Buffer
	for ctx.Err() == nil {
		input, err := socket.Recv(ctx)
		if err != nil {
			return fmt.Errorf("socket.Recv failed: %v", err)
		}
		buffer.WriteString(input)
		if !strings.HasSuffix(buffer.String(), "\r\n\r\n") {
			continue
		}
		ev, err := parseEvent(buffer.String())
		if err != nil {
			e.logger.Errorf("parseEvent failed: %v", err)
			continue
		}
		key := getKey(ev)
		ev["KEY"] = []string{key}

		e.PushEvent(event{key: key, data: ev})
		buffer.Reset()
	}

	return nil
}

func parseEvent(input string) (EventResponse, error) {
	resp := make(EventResponse)
	lines := strings.Split(input, "\r\n")
	for _, line := range lines {
		keys := strings.SplitAfterN(line, ":", 2)
		if len(keys) == 2 {
			key := strings.TrimSpace(strings.Trim(keys[0], ":"))
			value := strings.TrimSpace(keys[1])
			resp[key] = append(resp[key], value)
		} else if strings.Contains(line, "\r\n\r\n") || line == "" {
			return resp, nil
		}
	}
	return resp, nil
}

func getKey(ev EventResponse) string {
	var key string
	switch ev.Get("Event") {
	case EventDialBegin:
		key = ev.Get("DestAccountCode")
	case EventDialEnd:
		key = ev.Get("DestAccountCode")
	case EventHangup:
		key = ev.Get("AccountCode")
	case EventOriginateResponse:
		key = ev.Get("ActionID")
	}

	return key
}
