package ami

import (
	"context"
	"errors"
	"fmt"
	"sync"
	"sync/atomic"
	"time"

	"github.com/sirupsen/logrus"
	goami "gitlab.atsaero.ru/go-packages/goami/ami"
)

const (
	dialTimeout = time.Second * 10 // таймаут соединения с астериском
	liveTime    = time.Minute * 30 // время жизни воркера
)

// commandFunc тип функции-обработчика ami команды
type commandFunc func(*goami.Socket) error

// connPool пул соединений
type connPool struct {
	m             sync.Mutex
	workersAmount uint
	pool          map[string]*poolItem
	ctx           context.Context
	logger        *logrus.Entry
	user          string
	secret        string
}

// callbackParams структура для передачи callback функции в eventPool
type callbackParams struct {
	key string
	f   CallbackEventFunc
}

type ConnPool interface {
	launchCommands(addr string, cbs ...*callbackParams) func(commandFunc) error
	startPoolItem(p *poolItem, addr string)
}

// poolItem элемент пула отвечающий за соединения к одному адресу
type poolItem struct {
	ch        chan commandFunc
	wr        *atomic.Int32
	ctx       context.Context
	cancel    context.CancelFunc
	eventPool *eventPool
	logger    *logrus.Entry
}

func NewConnPool(ctx context.Context, user, secret string, workersAmount uint, logger *logrus.Entry) *connPool {
	c := &connPool{
		pool:          make(map[string]*poolItem),
		ctx:           ctx,
		user:          user,
		secret:        secret,
		workersAmount: workersAmount,
		logger:        logger.WithField("model", "pool"),
	}

	return c
}

func newPoolItem(ctx context.Context, addr, user, secret string, logger *logrus.Entry) *poolItem {
	ctx, cancel := context.WithCancel(ctx)
	return &poolItem{
		ctx:       ctx,
		cancel:    cancel,
		ch:        make(chan commandFunc, 100),
		wr:        &atomic.Int32{},
		eventPool: NewEventPool(ctx, addr, user, secret, logger),
		logger:    logger,
	}
}

// launchCommands возвращает функцию-приемник для обработчика команды и коллбэков событий
func (c *connPool) launchCommands(addr string, cbs ...*callbackParams) func(commandFunc) error {
	c.m.Lock()
	poolItem, ok := c.pool[addr]
	if !ok {
		poolItem = newPoolItem(c.ctx, addr, c.user, c.secret, c.logger)
		c.pool[addr] = poolItem
	}
	c.m.Unlock()

	// добавляем отложенные функции в обработчик
	for _, cb := range cbs {
		poolItem.eventPool.AddCallback(cb.key, cb.f)
	}

	// поднимаем воркеры, если нужно
	c.startPoolItem(poolItem, addr)

	return func(fn commandFunc) error {
		if fn == nil {
			return nil
		}
		t := time.NewTimer(time.Millisecond * 500)
		select {
		case <-poolItem.ctx.Done():
			return errors.New("context closed")
		case poolItem.ch <- fn:
		case <-t.C:
			return errors.New("channel is blocked")
		}

		return nil
	}
}

// startPoolItem запускает воркеры элемента пула
func (c *connPool) startPoolItem(p *poolItem, addr string) {
	needToStart := c.workersAmount - uint(p.wr.Load())
	for i := 1; i <= int(needToStart); i++ {
		go func(i int) {
			if err := p.worker(p.ctx, p.ch, addr, c.user, c.secret, int32(i), c.logger); err != nil {
				c.logger.Errorf("worker [%d] %s failed: %v", i, addr, err)
				p.cancel()
				c.m.Lock()
				delete(c.pool, addr)
				c.m.Unlock()
			}
		}(i)
	}
}

// worker процесс-обработчик соединения
func (p *poolItem) worker(ctx context.Context, commands chan commandFunc, addr, user, secret string, num int32, logger *logrus.Entry) error {
	p.wr.Add(1)
	defer p.wr.Add(-1)

	logger = logger.WithField("addr", addr)

	logger.Debugf("start worker [%d] %s", num, addr)
	defer logger.Debugf("stopped worker [%d] %s", num, addr)

	ctx, cancel := context.WithCancel(ctx)
	defer cancel()

	liveTimer := time.NewTimer(liveTime)

	go func() {
		<-liveTimer.C
		cancel()
	}()

	connTimer := time.NewTimer(dialTimeout)
	connected := make(chan struct{})

	// реализация dialTimeout для ami.NewSocket
	go func() {
		select {
		case <-connTimer.C:
			cancel()
			p.logger.Error("connect timeout")
		case <-connected:
		}
	}()

	uuid, err := goami.GetUUID()
	if err != nil {
		return fmt.Errorf("ami.GetUUID failed: %v", err)
	}

	socket, err := goami.NewSocket(ctx, addr)
	if err != nil {
		return fmt.Errorf("ami.NewSocket failed: %v", err)
	}
	defer socket.Close(ctx)

	if err := goami.Login(ctx, socket, user, secret, "Off", uuid); err != nil {
		return fmt.Errorf("ami.Login failed: %v", err)
	}

	close(connected)

	logger.Infof("worker [%d] %s connected and ready", num, addr)

loop:
	for {
		select {
		case <-ctx.Done():
			logger.Info("context Done")
			break loop
		case comm, ok := <-commands:
			if !ok {
				p.logger.Error("command channel is closed")
				break loop
			}

			liveTimer.Reset(liveTime)

			if comm == nil {
				p.logger.Error("command is nil")
				break loop
			}
			if err := comm(socket); err != nil {
				break loop
			}
		}
	}
	goami.Logoff(ctx, socket, uuid+`out`)

	return nil
}
